import bpy

# 根据节点的显示名称或ShaderNodeGroup的内部名称查找节点
def find_node_by_name_or_group_name(nodes, target_name):
    # 首先尝试根据节点的显示名称查找
    node = next((node for node in nodes if node.name == target_name), None)
    # 如果没有找到，再尝试根据ShaderNodeGroup的内部名称查找
    if not node:
        node = next((node for node in nodes if node.type == 'GROUP' and node.node_tree and node.node_tree.name == target_name), None)
    return node


# 连接或断开两个节点
def node_ops(a_node_group_name, b_node_name, ops_type):
    for obj in bpy.context.selected_objects:
        if obj.type == 'MESH' and obj.material_slots:
            for slot in obj.material_slots:
                if slot.material:
                    material = slot.material
                    if material and material.node_tree:
                        nodes = material.node_tree.nodes
                        links = material.node_tree.links

                        # 查找具有指定node_tree.name的ShaderNodeGroup实例
                        a_node = find_node_by_name_or_group_name(nodes, a_node_group_name)
                        # b_node保持不变，依据名称直接查找
                        b_node = nodes.get(b_node_name)

                        if a_node and b_node:
                            if ops_type == 'c':
                                try:
                                    links.new(a_node.outputs[0], b_node.inputs[0])
                                    print(f"Connected {a_node_group_name} to {b_node_name}.")
                                except Exception as e:
                                    print(f"Failed to connect {a_node_group_name} to {b_node_name}: {e}")
                            elif ops_type == 'd':
                                for link in list(links):
                                    if link.from_node == a_node and link.to_node == b_node:
                                        links.remove(link)
                                print(f"Disconnected {a_node_group_name} from {b_node_name}.")
                            else:
                                print("ERROR: OPS TYPE NOT FOUND")
                        else:
                            if not a_node:
                                print(f"Node group {a_node_group_name} not found.")
                            if not b_node:
                                print(f"Node {b_node_name} not found.")
                    else:
                        print("No active material or node tree found.")
                else:
                    pass
            else:
                print("No material slots found.")




# 在兩個節點之間插入Shader Node Group
def insert_shader_between_nodes(material, a_node_name, b_node_name, shader_name):

    for obj in bpy.context.selected_objects:
        if obj.type == 'MESH' and obj.material_slots:
            for slot in obj.material_slots:
                material = slot.material
                if material and material.node_tree:
                    nodes = material.node_tree.nodes
                    links = material.node_tree.links
                    # 查找节点
                    a_node = find_node_by_name_or_group_name(nodes, a_node_name)
                    b_node = nodes.get(b_node_name)
                    shader_node = find_node_by_name_or_group_name(nodes, shader_name)

                    if not shader_node:
                        print(f"Shader node '{shader_name}' not found.")
                        return
                    # 检查shader_node的名字是否已经有"_N"后缀
                    if not shader_node.name.endswith("_N"):
                        shader_node.name += "_N"

                    if not a_node or not b_node:
                        missing_nodes = [name for name, node in [("a_node", a_node), ("b_node", b_node)] if not node]
                        print(f"Cannot find specified nodes: {', '.join(missing_nodes)}")
                        return

                    # 查找并断开a_node和b_node之间的连接
                    for link in links:
                        if link.from_node == a_node and link.to_node == b_node:
                            links.remove(link)
                            break  # 假设只有一个连接需要断开

                    # 建立新的连接
                    try:
                        # 假设连接到shader_node的合适输入和输出，这可能需要根据具体的shader_node进行调整
                        links.new(a_node.outputs[0], shader_node.inputs[0])  # 连接a_node到shader_node
                        links.new(shader_node.outputs[0], b_node.inputs[0])  # 连接shader_node到b_node
                        print(f"Inserted '{shader_name}' between {a_node_name} and {b_node_name}.")
                    except Exception as e:
                        print(f"Failed to insert '{shader_name}' between {a_node_name} and {b_node_name}: {e}")




# 用于在操作面板中显示的操作类
class NodeOps(bpy.types.Operator):
    bl_idname = "node.ops"
    bl_label = "Node Ops"
    bl_options = {'REGISTER', 'UNDO'}

    a_node_name: bpy.props.StringProperty(name="A Node Name") # type: ignore
    b_node_name: bpy.props.StringProperty(name="B Node Name") # type: ignore
    ops_type: bpy.props.StringProperty(name="Ops Type") # type: ignore

    def execute(self, context):
        node_ops(self.a_node_name, self.b_node_name, self.ops_type)
        return {'FINISHED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)

    def node_ops(a_node_name, b_node_name, ops_type):
        obj = bpy.context.active_object
        if obj.material_slots:
            material = obj.active_material
            if material and material.node_tree:
                nodes = material.node_tree.nodes
                links = material.node_tree.links

                a_node = nodes.get(a_node_name)
                b_node = nodes.get(b_node_name)

                if a_node and b_node:
                    if ops_type == 'c':
                        try:
                            links.new(a_node.outputs[0], b_node.inputs[0])
                            print(f"Connected {a_node_name} to {b_node_name}.")
                        except Exception as e:
                            print(f"Failed to connect {a_node_name} to {b_node_name}: {e}")
                    elif ops_type == 'd':
                        for link in list(links):
                            if link.from_node == a_node and link.to_node == b_node:
                                links.remove(link)
                        print(f"Disconnected {a_node_name} from {b_node_name}.")
                    else:
                        print("ERROR: OPS TYPE NOT FOUND")
                else:
                    if not a_node:
                        print(f"Node {a_node_name} not found.")
                    if not b_node:
                        print(f"Node {b_node_name} not found.")
            else:
                print("No active material or node tree found.")
        else:
            print("No material slots found.")


    def draw(self, context):
        layout = self.layout
        layout.prop(self, "a_node_name")
        layout.prop(self, "b_node_name")
        layout.prop(self, "ops_type")



def add_color_attribute(name=None, color=None, ops=None):
    # Get all selected objects
    selected_obj = bpy.context.selected_objects
    for obj in selected_obj:
        # Check if the object is a mesh
        if obj.type != 'MESH':
            print(f"Skipped {obj.name} because it is not a mesh.")
            continue
        
        if ops == "R":
            if name in obj.data.vertex_colors:
                obj.data.vertex_colors.remove(obj.data.vertex_colors[name])
                # Update the mesh
                obj.data.update()
        elif ops == "A":
            if name not in obj.data.vertex_colors:
                obj.data.vertex_colors.new(name=name)
            color_layer = obj.data.vertex_colors[name]
            for poly in obj.data.polygons:
                for loop_index in poly.loop_indices:
                    color_layer.data[loop_index].color = color
        else:
            return
# add_color_attribute("Shaders_Mask_4896", (1, 1, 1, 1),"A") # Name , (R,G,B,A) , ops = "A" for add and "R" for remove

# 調整節點位置
def adjust_node_positions(material):
    nodes = material.node_tree.nodes
    links = material.node_tree.links

    # 假设已经有了Reroute节点和Shader Node Group的引用
    reroute1 = nodes.get("19_sfgw89_Reroute_us64_001")
    reroute2 = nodes.get("19_sfgw89_Reroute_us64_002")
    group_node = next((node for node in nodes if node.type == 'GROUP' and node.get("NodeFX_Tag") == True), None)  # 假设只有一个节点组
    output_node = next((node for node in nodes if node.type == 'OUTPUT_MATERIAL'), None)

    if group_node and output_node:
        # 设置group_node的位置
        group_node.location.x = output_node.location.x - output_node.width - 200  # 200为X轴上的间距
        group_node.location.y = output_node.location.y


    if reroute1 and group_node:
        # 设置reroute1的位置
        reroute1.location.x = group_node.location.x - 100  # 在group_node左侧100单位
        reroute1.location.y = group_node.location.y - 40  # 在group_node上方100单位

    if reroute2 and group_node:
        # 设置reroute2的位置
        reroute2.location.x = group_node.location.x + 200  # 在group_node左侧100单位
        reroute2.location.y = group_node.location.y - 20  # 在group_node下方100单位

def connect_reroute_to_active_output():
    # 獲取當前選擇的物體
    selected_objects = [obj for obj in bpy.context.selected_objects if obj.type == 'MESH']
    
    # 遍歷每個選擇的物體
    for obj in selected_objects:
        # 遍歷每個物體的材質
        for mat in obj.data.materials:
            if mat:
                nodes = mat.node_tree.nodes
                # 尋找類型為'Material Output'的節點，而不是依賴名稱
                material_outputs = [node for node in nodes if node.type == 'OUTPUT_MATERIAL']

                if not material_outputs:
                    print(f'Material "{mat.name}" on Object "{obj.name}" does not have a Material Output node.')
                    continue
                
                # 選擇第一個材料輸出節點作為目標節點
                material_output = material_outputs[0]

                # 獲取名稱為'Reroute_2'的reroute節點
                reroute_node = nodes.get('19_sfgw89_Reroute_us64_002')
                
                if reroute_node:
                    # 獲取reroute節點的輸出
                    reroute_output = reroute_node.outputs[0]  # 假設reroute節點只有一個輸出
                    
                    # 創建連接到材質輸出的Surface插槽
                    links = mat.node_tree.links
                    new_link = links.new(reroute_output, material_output.inputs['Surface'])
                    
                    pass
                else:
                    pass
            else:
                pass
